"use strict";
var tslib_1 = require("tslib");
var crypto_1 = (0, tslib_1.__importDefault)(require("crypto"));
var fs_1 = (0, tslib_1.__importDefault)(require("fs"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var test_at_scale_core_1 = require("@lambdatest/test-at-scale-core");
var constants_1 = require("./constants");
var JestReporter = /** @class */ (function () {
    function JestReporter(globalConfig) {
        this.testSuiteResults = new Map();
        this._globalConfig = globalConfig;
        this.testResults = [];
        this.filename = "";
        this.timings = [];
        this.tIndex = 0;
    }
    JestReporter.prototype.onTestStart = function (test) {
        this.filename = test.path;
        this.tIndex = 0;
    };
    JestReporter.prototype.onTestResult = function (test, testResult) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var CODE_COVERAGE_DIR, filename, coverageFileName;
            var _this = this;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        CODE_COVERAGE_DIR = process.env.CODE_COVERAGE_DIR;
                        try {
                            this.timings = JSON.parse(fs_1.default.readFileSync(constants_1.TIMINGS_FILE, { encoding: "utf-8" }))
                                .map(function (dateStr) { return new test_at_scale_core_1.TASDate(dateStr); });
                            // eslint-disable-next-line @typescript-eslint/no-explicit-any
                        }
                        catch (err) {
                            this.timings = [];
                        }
                        filename = test.path;
                        // append blocked testResults
                        testResult.testResults.forEach(function (jestTestResult) {
                            var locator = test_at_scale_core_1.Util.getLocator(filename, jestTestResult.ancestorTitles, jestTestResult.title);
                            var blockTest = test_at_scale_core_1.Util.getBlockTestLocatorProperties(locator);
                            if (blockTest.isBlocked) {
                                _this.testResults.push(_this.toTASTestResult(jestTestResult, test_at_scale_core_1.Util.getTestStatus(blockTest.status)));
                            }
                            else {
                                _this.testResults.push(_this.toTASTestResult(jestTestResult, test_at_scale_core_1.Util.getTestStatus(jestTestResult.status)));
                            }
                        });
                        if (!(CODE_COVERAGE_DIR && testResult.coverage)) return [3 /*break*/, 2];
                        coverageFileName = CODE_COVERAGE_DIR + "/" + filename.replace(/\//g, '') + "/coverage-final.json";
                        fs_1.default.mkdirSync(path_1.default.dirname(coverageFileName), { recursive: true });
                        return [4 /*yield*/, test_at_scale_core_1.JSONStream.stringify(testResult.coverage, fs_1.default.createWriteStream(coverageFileName))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        this.filename = "";
                        return [2 /*return*/];
                }
            });
        });
    };
    JestReporter.prototype.onRunComplete = function (_, aggregatedResults) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var executionResult, code;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        executionResult = new test_at_scale_core_1.ExecutionResult(this.testResults, Array.from(this.testSuiteResults.values()));
                        fs_1.default.mkdirSync(path_1.default.dirname(constants_1.REPORT_FILE), { recursive: true });
                        return [4 /*yield*/, test_at_scale_core_1.JSONStream.stringify(executionResult, fs_1.default.createWriteStream(constants_1.REPORT_FILE))];
                    case 1:
                        _a.sent();
                        code = !aggregatedResults || aggregatedResults.success ? 0 : this._globalConfig.testFailureExitCode;
                        process.on('exit', function () {
                            if (typeof code === 'number' && code !== 0) {
                                process.exitCode = code;
                            }
                        });
                        return [2 /*return*/];
                }
            });
        });
    };
    JestReporter.prototype.toTASTestResult = function (testCaseResult, status) {
        var _a;
        var specStartTime = null;
        if ((status === test_at_scale_core_1.TestStatus.Passed || status === test_at_scale_core_1.TestStatus.Failed) && this.tIndex < this.timings.length) {
            // test-case ran (not skipped or blocked). Hence, will have an ordered entry in timings file
            specStartTime = this.timings[this.tIndex++];
        }
        var duration = (_a = testCaseResult.duration) !== null && _a !== void 0 ? _a : 0;
        var ancestorTitles = testCaseResult.ancestorTitles;
        var repoID = process.env.REPO_ID;
        var filename = this.filename;
        var testIdentifier = test_at_scale_core_1.Util.getIdentifier(filename, testCaseResult.title);
        var locator = test_at_scale_core_1.Util.getLocator(filename, ancestorTitles, testCaseResult.title);
        var suiteIdentifiers = ancestorTitles.map(function (suiteName) { return test_at_scale_core_1.Util.getIdentifier(filename, suiteName); });
        var blockTest = test_at_scale_core_1.Util.getBlockTestLocatorProperties(locator);
        var failureMessage = null;
        if (status === test_at_scale_core_1.TestStatus.Failed) {
            failureMessage = testCaseResult.failureMessages.join('\n\n');
        }
        var testResult = new test_at_scale_core_1.TestResult(crypto_1.default
            .createHash("md5")
            .update(repoID + "\n" + suiteIdentifiers.join("\n") + "\n" + testIdentifier)
            .digest("hex"), testIdentifier, testCaseResult.title, suiteIdentifiers.length > 0
            ? crypto_1.default
                .createHash("md5")
                .update(repoID + "\n" + suiteIdentifiers.join("\n"))
                .digest("hex")
            : null, locator, duration, status, blockTest.isBlocked, blockTest.source, specStartTime, failureMessage);
        for (var i = 0; i < ancestorTitles.length; i++) {
            var suiteTitle = ancestorTitles[i];
            var thisSuiteIdentifiers = suiteIdentifiers.slice(0, i + 1);
            var parentSuiteIdentifiers = thisSuiteIdentifiers.slice(0, -1);
            var suiteIdentifier = test_at_scale_core_1.Util.getIdentifier(filename, suiteTitle);
            var suiteLocator = test_at_scale_core_1.Util.getLocator(filename, ancestorTitles.slice(0, i), suiteTitle);
            var blockSuite = test_at_scale_core_1.Util.getBlockTestLocatorProperties(suiteLocator);
            var suiteID = crypto_1.default
                .createHash("md5")
                .update(repoID + "\n" + thisSuiteIdentifiers.join("\n"))
                .digest("hex");
            var suiteParams = this.getSuiteParams(suiteID, duration, status, specStartTime);
            if (blockSuite.isBlocked) {
                suiteParams.status = test_at_scale_core_1.Util.getTestStatus(blockSuite.status);
            }
            var testSuite = new test_at_scale_core_1.TestSuiteResult(suiteID, suiteIdentifier, parentSuiteIdentifiers.length > 0
                ? crypto_1.default
                    .createHash("md5")
                    .update(repoID + "\n" + parentSuiteIdentifiers.join("\n"))
                    .digest("hex")
                : null, suiteParams.duration, suiteParams.status, blockSuite.isBlocked, blockSuite.source, suiteParams.startTime);
            this.testSuiteResults.set(testSuite.suiteID, testSuite);
        }
        return testResult;
    };
    JestReporter.prototype.getSuiteParams = function (suiteID, testDuration, testStatus, testStartTime) {
        var existing = this.testSuiteResults.get(suiteID);
        if (!existing) {
            return {
                duration: testDuration,
                status: this.winnerSuiteStatus(test_at_scale_core_1.TestStatus.Passed, testStatus),
                startTime: testStartTime
            };
        }
        var suiteStartTime = null;
        if (existing.start_time && testStartTime) {
            suiteStartTime = new test_at_scale_core_1.TASDate(existing.start_time) < testStartTime
                ? new test_at_scale_core_1.TASDate(existing.start_time)
                : testStartTime;
        }
        else if (existing.start_time) {
            suiteStartTime = new test_at_scale_core_1.TASDate(existing.start_time);
        }
        else if (testStartTime) {
            suiteStartTime = new test_at_scale_core_1.TASDate(testStartTime);
        }
        return {
            duration: existing.duration + testDuration,
            status: this.winnerSuiteStatus(existing.status, testStatus),
            startTime: suiteStartTime
        };
    };
    JestReporter.prototype.winnerSuiteStatus = function (suiteStatus, testStatus) {
        if (suiteStatus === test_at_scale_core_1.TestStatus.BlockListed) {
            return test_at_scale_core_1.TestStatus.BlockListed;
        }
        if (suiteStatus === test_at_scale_core_1.TestStatus.Quarantined) {
            return test_at_scale_core_1.TestStatus.Quarantined;
        }
        if (suiteStatus === test_at_scale_core_1.TestStatus.Failed || testStatus === test_at_scale_core_1.TestStatus.Failed) {
            return test_at_scale_core_1.TestStatus.Failed;
        }
        if (suiteStatus === test_at_scale_core_1.TestStatus.Skipped && testStatus === test_at_scale_core_1.TestStatus.Skipped) {
            return test_at_scale_core_1.TestStatus.Skipped;
        }
        return test_at_scale_core_1.TestStatus.Passed;
    };
    return JestReporter;
}());
module.exports = JestReporter;
//# sourceMappingURL=jest-reporter.js.map